/*
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        28.05.2004
 Description:    Programm zur Ansteuerung eines Standart LCD
				 (HD44870),(SED1278F) und kompatible
*/

#ifndef _LCD_H_
 #define _LCD_H_

#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

//Prototypes
extern void lcd_write (char,char);
extern char lcd_read (char);
extern void lcd_init (void);
extern void lcd_clear (void);
extern void lcd_print_P (unsigned char,unsigned char,const char *Buffer,...);
extern void lcd_print_str (char *Buffer);
#define lcd_print(a,b,format, args...)  lcd_print_P(a,b,PSTR(format) , ## args)

//LCD_D0 - LCD_D3 connect to GND
//Im 4Bit Mode LCD_D4-->PORTx.0 ........ LCD_D7-->PORTx.3
//LCD_RS --> PORTx.4 | LCD_RW --> PORTx.5 | LCD_E --> PORTx.6 | PORTx.7-->NotConnect

//Anzahl der Zeilen 1,2 oder 4
//#define ONE_LINES					
#define TWO_LINES					
//#define THREE_LINES					
//#define FOUR_LINES					

#define LCD_Port_DDR			DDRA	//Port an dem das Display angeschlossen wurde
#define LCD_Port_Write			PORTA
#define LCD_Port_Read			PINA

#define LCD_RS					6 		//Pin fr RS
#define LCD_RW					5		//Pin fr Read/Write
#define LCD_E					4 		//Pin fr Enable
#define LCD_LED					7 		//Pin fr Hintergrund LED

#define LCD_DataOutput			0x0f
#define LCD_DataInput			0x00

#define BUSYBIT					7

#define NOP()	asm("nop")
#define WAIT() 	for (unsigned char b=0;b<200;b++){NOP();}

#define LCD_LED_ON	LCD_Port_Write |= (1<<LCD_LED);
#define LCD_LED_OFF	LCD_Port_Write &= ~(1<<LCD_LED);

#endif //_LCD_H_

